import requests

from . import itime


def __init__(params: dict):
    return IsddcAPI(**params)


class IsddcAPI:
    token_timeout_point = 0

    def __init__(
            self,
            *,
            api: str,
            keystone: str,
            username: str,
            password: str,
            tenant_id: str = None,
            tenant_name: str = None,
            http_timeout: int = 60,
            token_timeout: int = 60 * 59,
    ):
        self.api = api
        self.keystone = keystone
        self.username = username
        self.password = password
        self.http_timeout = http_timeout
        self.token_timeout = token_timeout

        if tenant_id:
            self.project_key = 'id'
            self.project_value = tenant_id
        else:
            self.project_key = 'name'
            self.project_value = tenant_name

        self.auth_api = f'http://{self.keystone}/v3/auth/tokens'

        self.headers = {
            'Content-Type': 'application/json',
            'X-Auth-Token': None
        }

    def __getattribute__(self, name):
        if name in ('get', 'post') and self.token_timeout_point < itime.Now.float:
            self.headers['X-Auth-Token']: str = requests.post(
                url=self.auth_api,
                json={'auth': {
                    'identity': {
                        'methods': ['password'],
                        'password': {'user': {
                            'domain': {'id': 'default'},
                            'name': self.username,
                            'password': self.password}}},
                    'scope': {'project': {
                        self.project_key: self.project_value,
                        'domain': {'id': 'default'}}}}}
            ).headers['X-Subject-Token']
            self.token_timeout_point = itime.Future(self.token_timeout).float

        return super().__getattribute__(name)

    def get(
            self,
            uri: str,
            params: dict = None,
            **kw
    ) -> requests.Response:
        return requests.get(
            url=f'http://{self.api}{uri}',
            params=params,
            headers=self.headers,
            timeout=self.http_timeout,
            **kw
        )

    def post(
            self,
            uri: str,
            data: dict = None,
            json: dict = None,
            **kw
    ) -> requests.Response:
        return requests.post(
            url=f'http://{self.api}{uri}',
            data=data,
            json=json,
            headers=self.headers,
            timeout=self.http_timeout,
            **kw
        )
