class SingletonMode:
    """Inherit me, you will
    become a singleton class."""
    _instance = None

    def __new__(cls, *a, **kw):
        if cls._instance is None:
            cls._instance = object.__new__(cls)
        return cls._instance


class DictMode:
    """Inherit me, You will have
    the grammar of the 'dict'."""

    def __getitem__(self, name):
        return self.__getattribute__(name)

    def __setitem__(self, name, value):
        self.__setattr__(name, value)

    def __delitem__(self, name):
        self.__delattr__(name)


class IterMode:
    """Inherit me, your object can be iterated,
    and can be called by the 'if in' statement."""

    def __iter__(self):
        yield from self


class ObjectMode(type):
    """To essentially turn a class into an object."""

    def __new__(mcs, cls, bases, attrs):
        return super().__new__(mcs, cls, bases, attrs)()


class FuncMode(type):
    """Enable classes to be called as functions.
    Instantiating class no longer get the object, it
    directly gets the return value of its __call__ method."""

    def __call__(cls, *a, **kw):
        obj = super().__call__(*a, **kw)

        assert hasattr(obj, '__call__'), \
            'Use FuncMode must define __call__ method.'

        return obj()
