from typing import Iterable


class IsddcDict(dict):
    """IsddcDict == dict
    I inherited 'dict', my main function is to
    allow the dict to get or set values by `d.key`.
    """

    def __init__(self, _data=None, **kw):
        for name, value in (_data or kw).items():
            self[name] = IsddcDict(value)

    def __new__(cls, _data={}, **kw):
        if isinstance(_data, dict):
            return dict.__new__(cls)

        if isinstance(_data, Iterable) and not isinstance(_data, str):
            return [IsddcDict(v) for v in _data]

        return _data

    def __getattribute__(self, name):
        """`self.name` to `self[name]`"""
        try:
            return super().__getattribute__(name)
        except AttributeError:
            return self.__getitem__(name)

    # def __getattribute__(self, name):
    #     """
    #     `self.name` to `self[name]`
    #     So let's find the Key, and then let's find the built-in method.
    #     """
    #     try:
    #         return super().__getitem__(name)
    #     except KeyError:
    #         try:
    #             return super().__getattribute__(name)
    #         except AttributeError:
    #             raise KeyError(name)

    def __setattr__(self, name, value):
        """`self.name = value` to `self[name] = value`"""
        self.__setitem__(name, value)

    def __delattr__(self, name):
        """`del self.name` to `del self[name]`"""
        self.__delitem__(name)

    def __deepcopy__(self, memo):
        """
        This method must be implemented,  otherwise
        `KeyError: '__deepcopy__'` will appear when
        `copy.deepcopy (IsddcDict obj)` is called.
        """
        return IsddcDict(self)

    def __getstate__(self):
        """This method is for compatibility pickle module."""
        return self

    def __setstate__(self, state):
        """This method is for compatibility pickle module."""
        pass

    def __hash__(self):
        return -1

    def __eq__(self, other):
        return super().__eq__(other)

    # This property is for compatibility abc.ABCMeta.
    __isabstractmethod__ = False
