import json
import threading as th

from pykafka import Topic
from pykafka import Producer
from pykafka import KafkaClient

from . import idec

__default__: Topic


@idec.TryExcept(Exception, mark='InitKafka')
@idec.Retry('InitKafka', count=10, cycle=10)
def __init__(params: dict) -> Topic or KafkaClient:
    topic: str = params.pop('topic', None)
    client = KafkaClient(**params)
    return client.topics[topic] if topic else KafkaClient


def send(msg: dict, *, topic: Topic = None):
    """Simple transmitter."""
    pro: Producer = (topic or __default__).get_sync_producer()
    pro.produce(json.dumps(msg).encode())

    # `pro.stop()` is very slow, so:
    th.Thread(
        target=lambda: pro.stop(),
        daemon=True,
        name='StopProducer'
    ).start()
