from kubernetes import client as kube_client

from . import idec
from . import OverFunc


@idec.TryExcept(Exception, mark='InitKube')
@idec.Retry('InitKube', count=10, cycle=10)
def __init__(params: dict) -> kube_client.CoreV1Api:
    kube_config = kube_client.Configuration()
    kube_config.host = f'{params["protocol"]}://{params["api_server"]}'
    kube_config.verify_ssl = params['verify_ssl']
    kube_config.api_key = {'authorization': 'Bearer ' + params['token'].strip()}

    api_client = kube_client.ApiClient(kube_config)
    OverFunc(api_client.close)
    kube_api = getattr(kube_client, params['load_apis'][0])(api_client)

    for api_str in params['load_apis'][1:]:
        api_obj = getattr(kube_client, api_str)(api_client)

        for method_str in dir(api_obj):
            method_obj = getattr(api_obj, method_str)

            if not method_str.startswith('_') and callable(method_obj):
                setattr(kube_api, method_str, method_obj)

    return kube_api


"""
apiVersion: v1
kind: ServiceAccount
metadata:
  name: oi-edge-agent
  namespace: kube-system

---

apiVersion: rbac.authorization.k8s.io/v1beta1
kind: ClusterRoleBinding
metadata:
  name: oi-edge-agent
roleRef:
  apiGroup: rbac.authorization.k8s.io
  kind: ClusterRole
  name: cluster-admin
subjects:
  - kind: ServiceAccount
    name: oi-edge-agent
    namespace: kube-system

kubectl get secret -n kube-system | grep oi-edge-agent | awk '{print $1}' | xargs kubectl describe secret -n kube-system | grep ^token

kubectl config view --minify | grep server
"""
