import re

try:
    import requests
except ModuleNotFoundError:
    pass

from . import ifs
from . import icmd
from . import ilog
from . import ibase
from . import idict


class Oxyry(metaclass=ibase.FuncMode):
    url = 'https://pyob.oxyry.com/obfuscate'
    headers = {
        'accept': '*/*',
        'accept-encoding': 'gzip, deflate, br',
        'accept-language': 'zh-CN,zh;q=0.9',
        'content-length': '4926',
        'content-type': 'application/json',
        'cookie': '_ga=GA1.2.649212281.1615279215; _gid=GA1.2.2137850909.1615279215',
        'origin': 'https://pyob.oxyry.com',
        'referer': 'https://pyob.oxyry.com/',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36'
    }
    data = {
        'append_source': False,
        'preserve': '',
        'remove_docstrings': True,
        'rename_default_parameters': False,
        'rename_nondefault_parameters': True,
        'source': None
    }

    def __init__(self, code: str):
        self.data['source'] = code

    def __call__(self) -> str:
        if 'requests' not in globals():
            raise ModuleNotFoundError(
                'Using this function requires the '
                'installation of the "requests" module.')

        ret = requests.post(
            self.url, json=self.data, headers=self.headers)

        assert ret.status_code == 200, \
            f'{ret.status_code}, {ret.text}'

        return ret.json()['dest']

    def __del__(self):
        del self.data['source']


class IsddcCodeObfuscer(metaclass=ibase.FuncMode):
    replace_symbols = {
        ' .': '.',
        ' )': ')',
        ' ,': ','
    }

    def __init__(
            self,
            source: str = None,
            target: str = None,
            *,
            hide_symbol: list = None
    ):
        self.source = source
        self.target = target
        self.hide_symbol = hide_symbol

    def __call__(self):
        raw_code: str = ifs.Filetor(self.source)
        oxyry_code: str = Oxyry(raw_code)
        oxyry_code: str = re.sub(r'^""', '', oxyry_code)
        oxyry_code: str = re.sub(r'\n +""#line:\d+\n', '\n', oxyry_code)

        ifs.Filetor(self.target, oxyry_code)
        icmd(f'autopep8 --in-place --aggressive {self.target}').raise_if_error()
        oxyry_pep8_code: str = ifs.Filetor(self.target)

        for a, b in self.replace_symbols.items():
            oxyry_pep8_code: str = oxyry_pep8_code.replace(a, b)

        oxyry_pep8_code_list: list = re.split(r' {2}# line:\d+\n', oxyry_pep8_code)
        compute_code = ''

        if self.hide_symbol:
            for code in oxyry_pep8_code_list[:-1]:
                compute_code += f'{code}\n# {self.hide_symbol}\n'
        else:
            compute_code = '\n'.join(oxyry_pep8_code_list)

        ifs.Filetor(self.target, compute_code)
        ilog.simple.info(f'Obfuscated module {ifs.basename(self.source)}')


def batch(
        source_dir: str,
        target_dir: str,
        *,
        hide_symbol: list = None
):
    source: idict = ifs.fetch_deep_path(source_dir)
    target: str = ifs.gendir(target_dir)

    for name in source:
        if name[-3:] != '.py':
            continue

        IsddcCodeObfuscer(
            source=source[name],
            target=ifs.abspath(target, name),
            hide_symbol=hide_symbol
        )
