import os
import time

import multiprocessing as mp

from multiprocessing.pool import Pool
from multiprocessing.pool import ThreadPool
from multiprocessing.pool import AsyncResult


class IsddcProcessPool(Pool):

    def __init__(
            self,
            name: str = None,
            *,
            processes: int = None,
            **kw
    ):
        name: str = name or 'IsddcProcess'
        processes: int = processes or (mp.cpu_count() or 1)

        super().__init__(
            processes=processes,
            initializer=self._initializer,
            initargs=[name],
            **kw)

        self.name = name
        self.processes = processes

        self.running = {}

    @staticmethod
    def _initializer(name: str):
        mp.current_process().name = name + str(os.getpid())

    def submit(self, func, *, flag: str = None, **kw) -> AsyncResult:
        task: AsyncResult = self.apply_async(
            func=func, kwds=kw,
            callback=kw.pop('callback', None),
            error_callback=kw.pop('error_callback', None))

        self.running.setdefault(flag, []).append(task)

        return task

    def block(
            self,
            flag: str = None,
            *,
            timeout: int = None,
            cycle: int or float = .1
    ) -> list:
        results = []

        try:
            running: list = self.running[flag]
        except KeyError:
            return results

        running_length = len(running)
        running_last_change_time = time.time()

        while running:
            time.sleep(cycle)

            for task in running.copy():
                if task.ready():
                    running.remove(task)

                    if task.successful() and task.get():
                        results.append(task.get())

            running_length_now = len(running)

            if timeout and running_length_now == running_length:
                if time.time() - running_last_change_time > timeout:
                    running.clear()
                    break
            else:
                running_length = running_length_now
                running_last_change_time = time.time()

        return results


class IsddcThreadPool(IsddcProcessPool, ThreadPool):
    ...
