import re
import time

from . import ibase


class Now(metaclass=ibase.ObjectMode):

    @property
    def int(self) -> int:
        return int(self.float)

    @property
    def float(self) -> float:
        return time.time()

    @property
    def struct(self) -> tuple:
        return time.localtime()

    @property
    def str(self) -> str:
        return self.strf('%F %T')

    @staticmethod
    def strf(format: str) -> str:
        return time.strftime(format)


class Future:

    def __init__(self, future_time: ...):
        if isinstance(future_time, (int, float)):
            self.future_time = future_time
        elif isinstance(future_time, str):
            self.future_time = Time2Second(future_time)
        else:
            raise TypeError(f'Unsupported parameter type: {type(future_time)}.')

    @property
    def int(self) -> int:
        return int(self.float)

    @property
    def float(self) -> float:
        return Now.float + self.future_time

    @property
    def struct(self) -> tuple:
        return time.localtime(self.float)

    @property
    def str(self) -> str:
        return self.strf('%F %T')

    def strf(self, format: str) -> str:
        return stamp2str(self.float, format)


class Past(Future):

    @property
    def float(self) -> float:
        return Now.float - self.future_time


def stamp2str(
        stamp: int or float = None,
        format: str = '%F %T',
        *,
        gmt: bool = False
) -> str:
    """timestamp to string time"""
    func = time.gmtime if gmt else time.localtime
    return time.strftime(format, func(stamp))


def str2stamp(
        string: str,
        format: str = '%Y-%m-%d %H:%M:%S'
) -> float:
    """string time to timestamp"""
    return time.mktime(time.strptime(string, format))


class Time2Second(metaclass=ibase.FuncMode):
    pattern = re.compile(r'''^
        (?:(\d+(?:\.\d+)?)d)?
        (?:(\d+(?:\.\d+)?)h)?
        (?:(\d+(?:\.\d+)?)m)?
        (?:(\d+(?:\.\d+)?)s)?
    $''', flags=re.X)

    minute = 60
    hour = 60 * minute
    day = 24 * hour

    def __init__(self, unit_time: str):
        self.unit_time = unit_time

    def __call__(self) -> int or float:
        d, h, m, s = self.pattern.findall(self.unit_time.lower())[0]
        d, h, m, s = self.g(d), self.g(h), self.g(m), self.g(s)
        return self.day * d + self.hour * h + self.minute * m + s

    def g(self, x: str) -> int or float:
        if not x:
            return 0
        try:
            return int(x)
        except ValueError:
            return float(x)


def second2time(sec: int or float, x: 'Not param' = '') -> str:
    """Pass in a time in seconds,
    and it will be converted to unit time."""
    if sec < 1:
        return f'{round(sec, 2)}s'

    m, s = divmod(round(sec), 60)
    h, m = divmod(m, 60)
    d, h = divmod(h, 24)

    if d:
        x += f'{d}d'
    if h:
        x += f'{h}h'
    if m:
        x += f'{m}m'
    if s:
        x += f'{s}s'

    return x
